package com.jiuqi.script.rhino;

import com.jiuqi.script.CompileException;
import com.jiuqi.script.EvalException;
import com.jiuqi.script.IScriptObject;
import com.jiuqi.script.Script;
import com.jiuqi.script.ScriptManager;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Calendar;
import java.util.Date;
import org.mozilla.javascript.BaseFunction;
import org.mozilla.javascript.Context;
import org.mozilla.javascript.Function;
import org.mozilla.javascript.FunctionObject;
import org.mozilla.javascript.IdFunctionObject;
import org.mozilla.javascript.IdScriptableObject;
import org.mozilla.javascript.JavaScriptException;
import org.mozilla.javascript.NativeJavaObject;
import org.mozilla.javascript.RhinoException;
import org.mozilla.javascript.Scriptable;
import org.mozilla.javascript.ScriptableObject;
import org.mozilla.javascript.Undefined;
import org.mozilla.javascript.UniqueTag;

public class RhinoScript implements Script {
   private Context cx = Context.enter();
   private Scriptable scope;

   static {
      ScriptManager.registerScriptFactory(new RhinoScriptFactory());
   }

   public RhinoScript() {
      this.scope = this.cx.initStandardObjects((ScriptableObject)null, false);
      this.regDefaultFunction();
      Context.exit();
   }

   public Object evaluate(String script, String sourceName) throws CompileException, EvalException {
      this.cx = Context.enter();

      Object var5;
      try {
         Object result = this.cx.evaluateString(this.scope, script, sourceName, 1, (Object)null);
         var5 = this.convertObject(result);
      } catch (JavaScriptException var9) {
         JavaScriptException e = var9;
         throw new EvalException(e.getMessage(), e.sourceName(), e.lineNumber(), e.columnNumber(), e);
      } catch (RhinoException var10) {
         RhinoException e = var10;
         throw new CompileException(e.getMessage(), e.sourceName(), e.lineNumber(), e.columnNumber(), e);
      } finally {
         Context.exit();
      }

      return var5;
   }

   public Object call(String name, Object[] args) throws NoSuchMethodException, CompileException, EvalException {
      Object[] param = args;
      if (param == null) {
         param = new Object[0];
      }

      this.convertArgument(param);
      this.cx = Context.enter();

      Object var7;
      try {
         Function fun = this.getFunctionFromScope(this.scope, name);
         if (fun == null) {
            throw new NoSuchMethodException("没有该函数:" + name);
         }

         Object result = fun.call(this.cx, this.scope, fun, param);
         var7 = this.convertObject(result);
      } catch (JavaScriptException var11) {
         JavaScriptException e = var11;
         throw new EvalException(e.getMessage(), e.sourceName(), e.lineNumber(), e.columnNumber(), e);
      } catch (RhinoException var12) {
         RhinoException e = var12;
         throw new CompileException(e.getMessage(), e.sourceName(), e.lineNumber(), e.columnNumber(), e);
      } finally {
         Context.exit();
      }

      return var7;
   }

   public void compile(String script, String sourceName) throws CompileException {
      this.cx = Context.enter();

      try {
         this.cx.evaluateString(this.scope, script, sourceName, 1, (Object)null);
      } catch (RhinoException var7) {
         RhinoException e = var7;
         throw new CompileException(e.getMessage(), e.sourceName(), e.lineNumber(), e.columnNumber(), e);
      } finally {
         Context.exit();
      }

   }

   public Object get(String name) {
      Object o = this.getValueFromScope(this.scope, name);
      return !(o instanceof UniqueTag) && !(o instanceof Undefined) ? o : null;
   }

   public void put(String name, Object value) {
      this.putValueToScope(this.scope, name, value);
   }

   public Object toJava(Object obj) {
      if (obj == null) {
         return null;
      } else if (obj instanceof RhinoScriptObject) {
         return ((RhinoScriptObject)obj).getScriptObject();
      } else if (!(obj instanceof IdScriptableObject)) {
         return obj;
      } else {
         IdScriptableObject result = (IdScriptableObject)obj;
         if (result.getClassName().equals("Date")) {
            IdFunctionObject v = (IdFunctionObject)ScriptableObject.getProperty(result, "getTime");
            Object m = result.execIdCall(v, Context.getCurrentContext(), this.scope, result, (Object[])null);
            long millis = ((Double)m).longValue();
            Calendar c = Calendar.getInstance();
            c.setTimeInMillis(millis);
            return c;
         } else {
            return result;
         }
      }
   }

   public Object toScript(Object obj) {
      if (obj == null) {
         return null;
      } else if (obj instanceof IScriptObject) {
         return new RhinoScriptObject((IScriptObject)obj);
      } else if (obj instanceof IdScriptableObject) {
         return obj;
      } else {
         if (obj instanceof NativeJavaObject) {
            Object result = ((NativeJavaObject)obj).unwrap();
            if (result instanceof IScriptObject) {
               return new RhinoScriptObject((IScriptObject)result);
            }

            if (result instanceof Calendar) {
               Calendar c = (Calendar)result;
               return Context.getCurrentContext().newObject(this.scope, "Date", new Object[]{new Long(c.getTimeInMillis())});
            }

            if (result instanceof Date) {
               Date d = (Date)result;
               return Context.getCurrentContext().newObject(this.scope, "Date", new Object[]{new Long(d.getTime())});
            }
         }

         return obj;
      }
   }

   private Function getFunctionFromScope(Scriptable scope, String name) {
      String parentName = name;
      Scriptable parent = scope;
      if (name == null) {
         return null;
      } else {
         Object o;
         while(name.indexOf(".") != -1) {
            parentName = parentName.substring(0, name.indexOf("."));
            o = parent.get(parentName, scope);
            if (!(o instanceof Scriptable)) {
               return null;
            }

            parent = (Scriptable)o;
            name = name.substring(name.indexOf(".") + 1);
         }

         o = parent.get(name, scope);
         return o instanceof Function ? (Function)o : null;
      }
   }

   private Object getValueFromScope(Scriptable scope, String name) {
      String parentName = name;
      Scriptable parent = scope;
      if (name == null) {
         return null;
      } else {
         while(name.indexOf(".") != -1) {
            parentName = parentName.substring(0, name.indexOf("."));
            Object o = parent.get(parentName, scope);
            if (!(o instanceof Scriptable)) {
               return null;
            }

            parent = (Scriptable)o;
            name = name.substring(name.indexOf(".") + 1);
         }

         return parent.get(name, scope);
      }
   }

   private void putValueToScope(Scriptable scope, String name, Object value) {
      String parentName = name;
      Scriptable parent = scope;
      if (name != null) {
         while(name.indexOf(".") != -1) {
            parentName = parentName.substring(0, name.indexOf("."));
            Object o = parent.get(parentName, scope);
            if (!(o instanceof Scriptable)) {
               return;
            }

            parent = (Scriptable)o;
            name = name.substring(name.indexOf(".") + 1);
         }

         if (value instanceof IScriptObject) {
            parent.put(name, scope, new RhinoScriptObject((IScriptObject)value));
         } else {
            parent.put(name, scope, value);
         }

      }
   }

   private void convertArgument(Object[] args) {
      if (args != null) {
         for(int i = 0; i < args.length; ++i) {
            if (args[i] instanceof Calendar) {
               Calendar c = (Calendar)args[i];
               args[i] = this.cx.newObject(this.scope, "Date", new Object[]{new Long(c.getTimeInMillis())});
            }
         }

      }
   }

   private Object convertObject(Object result) {
      if (result instanceof ScriptableObject) {
         ScriptableObject o = (ScriptableObject)result;
         if (o.getClassName().equals("CustomScriptObject")) {
            return ((RhinoScriptObject)o).getScriptObject();
         } else if (o.getClassName().equals("Date")) {
            BaseFunction v = (BaseFunction)ScriptableObject.getProperty(o, "getTime");
            Object m = v.call(this.cx, this.scope, o, (Object[])null);
            long millis = ((Double)m).longValue();
            Calendar c = Calendar.getInstance();
            c.setTimeInMillis(millis);
            return c;
         } else {
            return o;
         }
      } else if (result instanceof NativeJavaObject) {
         Object o = ((NativeJavaObject)result).unwrap();
         return o;
      } else {
         return result;
      }
   }

   private void regDefaultFunction() {
      if (this.scope != null) {
         Class c = ScriptHelper.class;
         ScriptHelper u = new ScriptHelper(this.scope);
         Method[] methods = c.getDeclaredMethods();

         for(int i = 0; i < methods.length; ++i) {
            if (!methods[i].getName().equals("getClassName") && Modifier.isPublic(methods[i].getModifiers())) {
               this.scope.put(methods[i].getName(), this.scope, new FunctionObject(methods[i].getName(), methods[i], u));
            }
         }

      }
   }
}
